from slippers.templatetags.slippers import register_components


def load_backoffice_slippers():
    register_components({
        "form_input": "components/forms/input.html",
        "form_input_code": "components/forms/input_code.html",
        "form_switch": "components/forms/switch.html",
        "form_switch_label": "components/forms/switch_label.html",
        "form_checkbox": "components/forms/checkbox.html",
        "form_quill": "components/forms/wysiwyg_input.html",
        "form_select2": "components/forms/select2.html",
        "form_dual_listbox": "components/forms/dual_listbox.html",
        "form_image": "components/forms/image.html",
        "form_checklist": "components/forms/checklist.html",
        "form_input_disabled": "components/forms/input_disabled.html",
        "form_select2_actions": "components/forms/select2_actions.html",
        "form_datetime": "components/forms/datetime.html",
        "form_price": "components/forms/price.html",

        "kt_section_title": "components/forms/kt_section_title.html",
        "kt_form_footer": "components/forms/kt_form_footer.html",
        "kt_separator": "components/forms/kt_separator.html",

        "simple_dategroup": "components/simple_forms/date_group.html",
        "simple_switch_label": "components/simple_forms/switch_label.html",
        "input_hidden": "components/simple_forms/input_hidden.html",
        "button": "components/simple_forms/button.html",

        "menu_item": "components/menu_item.html",
        "menu_category": "components/menu_category.html",
    })



from django.contrib.auth.models import Permission

from rest_framework import serializers


class PermissionSerializer(serializers.ModelSerializer):
    name = serializers.SerializerMethodField()

    class Meta:
        model = Permission
        fields = (
            "pk",
            "name"
        )

    def get_name(self, obj):
        return obj.__str__()



from dnoticias_backoffice import views

from django.urls import path


urlpatterns = [
    path("flagstate/", views.FlagStateListView.as_view(), name="flag-state-list-view"),
    path("flagstate/create/", views.FlagStateCreateView.as_view(), name="flag-state-create-view"),
    path("flagstate/<int:pk>/", views.FlagStateEditView.as_view(), name="flag-state-wildcard-view"),
    path("flagstate/delete/<int:pk>/", views.FlagStateDeleteView.as_view(), name="flag-state-delete-view"),

    path("group/", views.GroupListView.as_view(), name="group-list-view"),
    path("group/create/", views.GroupCreateView.as_view(), name="group-create-view"),
    path("group/<int:pk>/", views.GroupEditView.as_view(), name="group-detail-view"),
    path("group/delete/<int:pk>/", views.GroupDeleteView.as_view(), name="group-delete-view"),

    path("permissions/", views.PermissionsView.as_view(), name="permissions"),
    path("permissions/user/", views.UserPermissionsView.as_view(), name="permissions-per-user"),
    path("permissions/group/", views.GroupPermissionsView.as_view(), name="permissions-per-group"),
    path("groups/user/", views.UserGroupsView.as_view(), name="groups-per-user"),
    
    path('select2/users/', views.UsersSelect2View.as_view(), name="users-select2"),
    path('select2/permissions/', views.PermissionsSelect2View.as_view(), name="permissions-select2"),
]



from django.http import HttpRequest

from dnoticias_utils.templatetags.base import is_on

menu_items = []


def get_menu_items(request: HttpRequest) -> list:
    """Get the menu items. The menu items are registered using the register_menu_item function.

    :param request: The request object.
    :return: The menu items.
    :rtype: list
    """
    items = []

    for item in menu_items:
        permissions = item.get("permissions", [])
        active = any([is_on(request, url_name) for url_name in item.get("active_url_names", [])])
        has_permissions = request.user.has_perms(permissions) if permissions else True
        items.append({
            "name": item.get("name"),
            "icon": item.get("icon"),
            "url_name": item.get("url_name"),
            "active": active,
            "has_permissions": has_permissions,
            "is_category": item.get("is_category")
        })

    return items

def register_menu_item(
    name: str,
    icon: str = None,
    url_name: str = None,
    active_url_names: list = None,
    permissions: list = None,
    is_category: bool = False
):
    """Register a menu item appending it to the menu_items list.

    :parame name: The name of the menu item.
    :param icon: The icon of the menu item.
    :param url_name: The url name of the menu item.
    :param active_url_names: The url names that will be considered active.
    :param permissions: The permissions required to show the menu item.
    :param is_category: If the menu item is a category.
    :return: None
    """
    active_url_names = active_url_names or []
    menu_items.append({
        "name": name,
        "icon": icon,
        "url_name": url_name,
        "active_url_names": active_url_names,
        "permissions": permissions,
        "is_category": is_category
    })



from django.apps import AppConfig


class DnoticiasBackofficeConfig(AppConfig):
    name = 'dnoticias_backoffice'

    def ready(self):
        from .components import load_backoffice_slippers
        load_backoffice_slippers()



import os

from setuptools import find_packages, setup
from pathlib import Path

this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="dnoticias_backoffice",
    version='1.2.0a3',
    url="https://www.dnoticias.pt/",
    author="Pedro Mendes",
    author_email="pedro.trabalho.uma@gmail.com",
    maintainer="Nélson Gomes",
    maintainer_email="ngoncalves@dnoticias.pt",
    license="MIT",
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        'django',
        'django-flags',
        'djangorestframework',
        'django-dnoticias-tables',
        'slippers',
    ],
    include_package_data=True,
    packages=find_packages(),
    data_files=[('dnoticias_backoffice', [str(path) for path in Path('./dnoticias_backoffice/templates').rglob('*')  if os.path.isfile(str(path))])]
)



from cProfile import label
import logging
from typing import Optional

from django.contrib.auth.models import Permission, Group
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import get_user_model
from django import forms

from flags.conditions import get_conditions, get_condition
from flags.sources import get_flags
from flags.models import FlagState

logger = logging.getLogger(__name__)
User = get_user_model()


class FlagStateForm(forms.ModelForm):
    class Meta:
        model = FlagState
        fields = (
            "value",
            "required",
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        available_conditions_choices = [
            [condition, condition] for condition in sorted(get_conditions())
        ]
        self.fields["condition"] = forms.ChoiceField(
            label=_("Condição"),
            choices=available_conditions_choices,
        )

        available_name_choices = [
            [condition, condition] for condition in sorted(get_flags().keys())
        ]
        self.fields["name"] = forms.ChoiceField(
            label=_("Nome"),
            choices=available_name_choices,
        )

        if self.instance.pk:
            self.initial["condition"] = self.instance.condition
            self.initial["name"] = self.instance.name

    def clean(self):
        cleaned_data = super().clean()
        condition_name = cleaned_data.get("condition")
        value = cleaned_data.get("value")
        condition = get_condition(condition_name)
        validator = getattr(condition, "validate")

        if validator is not None:
            try:
                validator(value)
            except Exception as e:
                raise forms.ValidationError(e)

        cleaned_data["value"] = value
        return cleaned_data

    def save(self, *args, **kwargs):
        commit = kwargs.pop("commit", True)
        kwargs["commit"] = False

        self.instance = super().save(*args, **kwargs)
        self.instance.name = self.cleaned_data["name"]
        self.instance.condition = self.cleaned_data["condition"]

        if commit:
            self.instance.save()

        return self.instance


class PermissionForm(forms.Form):
    user = forms.ModelChoiceField(
        queryset=User.objects.all(),
        required=True
    )
    groups = forms.ModelMultipleChoiceField(
        queryset=Group.objects.all(),
        required=False
    )
    permissions = forms.ModelMultipleChoiceField(
        queryset=Permission.objects.none(),
        required=False
    )

    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)
        self.fields['permissions'].queryset = \
            Permission.objects.all().prefetch_related("content_type")

    def save(self):
        user = self.cleaned_data["user"]
        groups = self.cleaned_data["groups"]
        permissions = self.cleaned_data["permissions"]
        permissions_ids = [permission.id for permission in permissions]

        user.groups.set(groups)
        group_permissions_ids = []

        for group in groups:
            group_permissions_ids.extend([permission.id for permission in group.permissions.all()])

        permissions_ids = [
            permission_id for permission_id in permissions_ids 
            if permission_id not in group_permissions_ids
        ]

        user.user_permissions.set(permissions_ids)

        return user


class GroupForm(forms.ModelForm):
    permissions = forms.ModelMultipleChoiceField(
        queryset=Permission.objects.all().prefetch_related("content_type"),
        required=False,
    )

    class Meta:
        model = Group
        fields = (
            "name",
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if self.instance.pk:
            self.initial["permissions"] = \
                self.instance.permissions.all().prefetch_related("content_type")

    def save(self, commit: Optional[bool]=True):
        group: Group = super().save(commit=False)

        if commit:
            name = self.cleaned_data.get("name")
            group.name = name
            group.save()
            group.permissions.set(self.cleaned_data["permissions"])

        return group



from urllib.parse import urlparse

from django.http import HttpRequest
from django.conf import settings

from .menu import get_menu_items

def backoffice(request: HttpRequest) -> dict:
    """The backoffice context processor. This variables will be available in all templates.

    :param request: The request object.
    :return: The context variables.
    :rtype: dict
    """
    base_url = getattr(settings, "BASE_URL", "https://www.dnoticias.pt")
    site_domain = urlparse(base_url).netloc

    return {
        "BASE_URL": base_url,
        "SITE_DOMAIN": site_domain,
        "SITE_NAME": getattr(settings, "SITE_NAME", "Backoffice"),
        "MENU_ITEMS": get_menu_items(request),
    }
